//********************************************************************************
//
// This is the implementation for societies on Aeonian Dreams.
//
// Societies can be created in order for players to create their own rules to
// follow within some scope (i.e. the society). For instance, there could be a
// PKiller's society that allows all members to kill each other, or a Roleplayer's
// society where all members agree to roleplay whenever they are in the presence 
// of another member.
//
// Societies were implemented as a curiousity to see how players handle a world
// where they create and govern their own rules. Call me a sadist, but I'm 
// interested in what will happen...
//
// If this isn't your cup of tea, Societies can easily act as a fairly decent
// method for implementing clans. And, unlike most clan code out there, any given
// player can belong to any number of societies. If you decide to use our society
// code, please make note of it in your helpfile(s) relating to societies. Simply
// noting that this was orionally written by Alister of Aeonian Dreams would work
// splendid.
//
// Documentation for all of the public functions is contained in society.h
//
//  - Alister
//
//  Aeonian Dreams:
//    telnet://dreams.game-host.org:4000
//
//********************************************************************************

#include "conf.h"
#include "sysdep.h"

#include "structs.h"        // for char_data
#include "db.h"             // for LIB_ETC
#include "comm.h"           // for send_to_char
#include "interpreter.h"    // for ACMD()
#include "utils.h"          // for CREATE() and IDNUM()
#include "society.h"        // the interface we need to implement



/* Local variables */
int num_societies = 0;
struct society_data **society = NULL;





/* Local functions */
struct society_member *societyMemberFromList(const int id, struct society_member *list);
int fgetlinetomax(FILE *file, char *p, int maxlen);
bool societySave(const struct society_data *S, const char *filename);
struct society_data *societyLoad(const char *filename);
char *societyFilename(const struct society_data *S);
void  societyAdd(struct society_data *S);
void  societyRemove(struct society_data *S);
void  societyDelete(struct society_data *S);
struct society_data *societyGet(const char *name);




/* Structures */
struct society_member {

  struct society_member *next;
  int    id;

};


struct society_data {

  char *name;
  char *info;

  struct society_member *moderators;
  struct society_member *members;
  struct society_member *applicants;

  int  open_join;
  int  open_leave;

};

//********************************************************************************
// The implementation and documentation of local functions goes here
//********************************************************************************

//
// Return a pointer to the first member with the given ID
// return NULL if none exist.
//
struct society_member *societyMemberFromList(const int id, struct society_member *list) {

  for(; list != NULL; list = list->next)
    if(id == list->id)
      return list;

  return list; // NULL
}


void writeSocietyMasterlist() {
 
  int i;
  FILE *fl;
  char buf[MAX_STRING_LENGTH];
  
  if(!(fl = fopen(SOCIETY_LIST, "w+"))) {
    log("ERROR: could not open society masterlist for writing.");
    return;
  }


  sprintf(buf, "%d\n", num_societies); 
  fprintf(fl, buf);
  for(i = 0; i < num_societies; i++) {
    fprintf(fl, "%s%d.soc\n", LIB_SOCIETY, i);
  }

  fclose(fl);
}


//
// fill up the buffer with characters until
// a newline is reached, or we hit our critical
// length. Return how many characters were read
//
int fgetlinetomax(FILE *file, char *p, const int maxlen)
{
  int count = 0;

  while(!feof(file) && count < maxlen - 1) {

    p[count] = fgetc(file);
    if(p[count] == '\n')
      break;

    count++;
  }

  p[count] = '\0';
  return count;
}


//
// return a copy of where the society should be saved to
//
char *societyFilename(const struct society_data *S) {

  int i;

  for(i = 0; i < num_societies; i++)
    if(!strcmp(S->name, society[i]->name))
      break;

  if(i == num_societies)
    return NULL;
  else {
    char buf[MAX_STRING_LENGTH]; // I doubt we'll ever have a billion societies ...
    sprintf(buf, "%s%d.soc", LIB_SOCIETY, i);
    return strdup(buf);
  }
}


//
// Save the given society to the given filename
// See societySave for the format of society files
//
struct society_data *societyLoad(const char *filename) {


  FILE *fl;
  char line[MAX_STRING_LENGTH];
  char *info;
  int  id, infolen;
  struct society_data *S;

  if(filename == NULL) {
    log("ERROR: passed null pointer to societyLoad");
    return NULL;
  }

  if( !(fl = fopen(filename, "r")) ) {
    log("ERROR: could not open file, %s, in societyLoad.", filename);
    return NULL;
  }


  CREATE(S, struct society_data, 1);


  fgetlinetomax(fl, line, MAX_STRING_LENGTH);
  sscanf(line, "%d %d", &(S->open_join), &(S->open_leave));
  

  fgetlinetomax(fl, line, MAX_STRING_LENGTH);
  S->name = strdup(line);


  // load moderators
  while(TRUE) {

    struct society_member *moderator;
    fgetlinetomax(fl, line, MAX_STRING_LENGTH);
    if(!strcmp(line, "~"))
      break;

    sscanf(line, "%d", &id);
    CREATE(moderator, struct society_member, 1);
    moderator->id = id;
    moderator->next = S->moderators;
    S->moderators = moderator;
  }


  // load members
  while(TRUE) {

    struct society_member *member;
    fgetlinetomax(fl, line, MAX_STRING_LENGTH);
    if(!strcmp(line, "~"))
      break;

    sscanf(line, "%d", &id);
    CREATE(member, struct society_member, 1);
    member->id = id;
    member->next = S->members;
    S->members = member;
  }


  // load applicants
  while(TRUE) {

    struct society_member *applicant;
    fgetlinetomax(fl, line, MAX_STRING_LENGTH);
    if(!strcmp(line, "~"))
      break;

    sscanf(line, "%d", &id);
    CREATE(applicant, struct society_member, 1);
    applicant->id = id;
    applicant->next = S->applicants;
    S->applicants = applicant;
  }


  infolen = 0;
  info = strdup("");
  // keep on taking in strings until we hit a line
  // that contains a single ~
  strcpy(line, "");
  S->info = fread_string(fl, line);

  if(strlen(line) > 0) {
    // we should do something here ...
  }


  fclose(fl);
  return S;
}


//
// Save the given society to the given filename and the format:
//
// <open_join> <open_leave>
// <society name>
// <moderator 1>
// <moderator 2>
// ...
// ~
// <member 1>
// <member 2>
// ...
// ~
// <applicant 1>
// <applicant 2>
// ...
// <society info ....
//  ...>
// ~
//
bool societySave(const struct society_data *S, const char *filename) {

  FILE *fl;
  struct society_member *list;

  if(filename == NULL) {
    log("ERROR: passed null pointer to societySave when saving %s", S->name);
    return FALSE;
  }

  if( !(fl = fopen(filename, "w+")) ) {
    log("ERROR: could not save society, %s, to filename, %s.", S->name, filename);
    return FALSE;
  }

  fprintf(fl, "%d %d\n", S->open_join, S->open_leave);
  fprintf(fl, "%s\n", S->name);

  for(list = S->moderators; list != NULL; list = list->next)
    fprintf(fl, "%d\n", list->id);
  fprintf(fl, "~\n");

  for(list = S->members; list != NULL; list = list->next)
    fprintf(fl, "%d\n", list->id);
  fprintf(fl, "~\n");

  for(list = S->applicants; list != NULL; list = list->next)
    fprintf(fl, "%d\n", list->id);
  fprintf(fl, "~\n");

  fprintf(fl, "%s~\n", S->info);

  fclose(fl);
  return TRUE;
}


//
// free the society and everything it points to
//
void societyDelete(struct society_data *S) {

  struct society_member *next, *member;

  if(S->moderators)
    for(member = S->moderators; member != NULL; member = next) {
      next = member->next;
      free(member);
    }

  if(S->members)
    for(member = S->members; member != NULL; member = next) {
      next = member->next;
      free(member);
    }

  if(S->applicants)
    for(member = S->applicants; member != NULL; member = next) {
      next = member->next;
      free(member);
    }

  free(S->name);
  free(S->info);
  free(S);
}


//
// Remove a society from the list of all societies,
// and then free up its memory
//
void societyRemove(struct society_data *S) {

  int i, j;
  for(i = 0; i < num_societies; i++)
    if(society[i] == S)
      break;


  if(i == num_societies) {
    log("ERROR: tried to remove society, %s, which did not formally exist.", S->name);
    societyDelete(S);
    return;
  }


  num_societies--;
  for(j = i; j < num_societies; j++)
    society[j] = society[j+1];
  for(; i < num_societies; i++)
    societySave(society[i], societyFilename(society[i]));

  societyDelete(S);
  writeSocietyMasterlist();
}


//
// Add a new society to the list of all societies
//
void societyAdd(struct society_data *S) {

  int i;
  struct society_data **oldList = society;

  society = malloc(sizeof(struct society_data *) * (num_societies + 1) );

  for(i = 0; i < num_societies; i++)
    society[i] = oldList[i];

  society[num_societies] = S;
  num_societies++;
  societySave(S, societyFilename(S));
  free(oldList);
  writeSocietyMasterlist();
}


//
// return a pointer to the society with the given name
// return NULL if none exist
//
struct society_data *societyGet(const char *name) {

  int i;
  char *newname = strlwr(strdup(name));

  for(i = 0; i < num_societies; i++)
    if(!strcmp(newname, strlwr(strdup(society[i]->name)))) {
      free(newname);
      return society[i];
    }

  free(newname);
  return NULL;
}




//********************************************************************************
// Here are the functions we need to implement from society.h
//********************************************************************************

bool societyReload(const char *name) {

  int i;
  struct society_data *S;

  if(!(S = societyGet(name)))
    return FALSE;

  for(i = 0; i < num_societies; i++)
    if(S == society[i]) {
      char buf[MAX_STRING_LENGTH];
      societyDelete(society[i]);
      sprintf(buf, "%s%d.soc", LIB_SOCIETY, i);
      society[i] = societyLoad(buf);
      return TRUE;
    }

  return FALSE;
}


void societyBoot() {

  FILE *fl;
  int i, len;
  char line[MAX_STRING_LENGTH];

  if(!(fl = fopen(SOCIETY_LIST, "r+"))) {
    log("  Could not open society masterlist. Aborting.");
    return;
  }

  if(feof(fl)) {
    log("  Society masterlist contained no data! Aborting.");
    return;
  }

  len = fgetlinetomax(fl, line, MAX_STRING_LENGTH);
  sscanf(line, "%d", &num_societies);
  if(num_societies <= 0) {
    log("  No societies have formed yet.");
    society = NULL;
    return;
  }

  society = malloc( sizeof(struct society_data *) * num_societies );

  for(i = 0; i < num_societies; i++) {
    if( (len = fgetlinetomax(fl, line, MAX_STRING_LENGTH)) > 0) {
      log("  Loading society: %s", line);
      society[i] = societyLoad(line);
    }
    else {
      log("  Found blank line while looking for society names. Aborting.");
      for(i--; i >= 0; i--)
	societyDelete(society[i]);
      free(society);  // ...it would be nice, wouldn't it?
      society = NULL;
      num_societies = 0;
      fclose(fl);
      return;
    }
  }
  fclose(fl);
}


bool isSociety(const char *name) {
  return (societyGet(name) != NULL);
}


bool societyCreate(const char *name) {

  struct society_data *S;

  if(isSociety(name))
    return FALSE;

  CREATE(S, struct society_data, 1);
  S->name = strdup(name);
  S->info = strdup(DEFAULT_SOCIETY_INFO);
  S->moderators = NULL;
  S->members = NULL;
  S->applicants = NULL;

  societyAdd(S);
  return TRUE;
}


void societyDestroy(const char *name) {

  struct society_data *S = societyGet(name);
  if(S)  societyRemove(S);
}


bool societyApply(const char *name, const struct char_data *ch) {

  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return FALSE;

  if(societyMemberFromList(GET_IDNUM(ch), S->moderators) ||
     societyMemberFromList(GET_IDNUM(ch), S->members))
    return FALSE;

  if(societyMemberFromList(GET_IDNUM(ch), S->applicants))
    return TRUE;

  struct society_member *new;
  CREATE(new, struct society_member, 1);
  new->id = GET_IDNUM(ch);

  new->next = S->applicants;
  S->applicants = new;
  societySave(S, societyFilename(S));

  return TRUE;
}


bool societyInduct(const char *name, const struct char_data *ch) {

  struct society_member *m, *temp;
  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return FALSE;

  if(societyMemberFromList(GET_IDNUM(ch), S->moderators) ||
     societyMemberFromList(GET_IDNUM(ch), S->members))
    return TRUE;

  if( (m = societyMemberFromList(GET_IDNUM(ch), S->applicants)) ) {
    REMOVE_FROM_LIST(m, S->applicants, next);
    free(m);
  }

  CREATE(m, struct society_member, 1);
  m->id = GET_IDNUM(ch);
  m->next = S->members;
  S->members = m;
  societySave(S, societyFilename(S));
  return TRUE;
}


bool societyMakeModerator(const char *name, const struct char_data *ch) {

  struct society_member *m, *temp;
  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return FALSE;

  if(societyMemberFromList(GET_IDNUM(ch), S->moderators))
    return TRUE;

  if( (m = societyMemberFromList(GET_IDNUM(ch), S->members)) ) {
    REMOVE_FROM_LIST(m, S->members, next);
    free(m);
  }
  else if( (m = societyMemberFromList(GET_IDNUM(ch), S->applicants)) ) {
    REMOVE_FROM_LIST(m, S->applicants, next);
    free(m);
  }
    
  CREATE(m, struct society_member, 1);
  m->id = GET_IDNUM(ch);
  m->next = S->moderators;
  S->moderators = m;
  societySave(S, societyFilename(S));
  return TRUE;
}


void societyExpel(const char *name, const struct char_data *ch) {

  struct society_member *m, *temp;
  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return;

  if( (m = societyMemberFromList(GET_IDNUM(ch), S->moderators)) ) {
    REMOVE_FROM_LIST(m, S->moderators, next);
    free(m);
  }
  else if( (m = societyMemberFromList(GET_IDNUM(ch), S->members)) ) {
    REMOVE_FROM_LIST(m, S->members, next);
    free(m);
  }
  societySave(S, societyFilename(S));
}


void societyDecline(const char *name, const struct char_data *ch) {

  struct society_member *m, *temp;
  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return;

  if( (m = societyMemberFromList(GET_IDNUM(ch), S->applicants)) ) {
    REMOVE_FROM_LIST(m, S->applicants, next);
    free(m);
  }
  societySave(S, societyFilename(S));
}


bool societyIsMember(const char *name, const struct char_data *ch) {

  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return FALSE;

  if(societyMemberFromList(GET_IDNUM(ch), S->moderators) || 
     societyMemberFromList(GET_IDNUM(ch), S->moderators))
    return TRUE;
  else
    return FALSE;
}


bool societyIsModerator(const char *name, const struct char_data *ch) {

  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return FALSE;

  return (societyMemberFromList(GET_IDNUM(ch), S->moderators) != NULL);
}


bool societyIsApplicant(const char *name, const struct char_data *ch) {

  struct society_data *S = societyGet(name);

  if(S == NULL || IS_NPC(ch))
    return FALSE;

  return (societyMemberFromList(GET_IDNUM(ch), S->applicants) != NULL);
}


bool societyOpenJoin(const char *name) {

  struct society_data *S = societyGet(name);
  return (S != NULL && S->open_join == TRUE);
}


bool societyOpenLeave(const char *name) {

  struct society_data *S = societyGet(name);
  return (S != NULL && S->open_leave == TRUE);
}


bool societySetOpenJoin(const char *name, const int val) {

  struct society_data *S = societyGet(name);
  if(S == NULL)
    return FALSE;
 
  if(val == FALSE)    S->open_join = FALSE;
  else                S->open_join = TRUE;

  societySave(S, societyFilename(S));
  return TRUE;
}


bool societySetOpenLeave(const char *name, const int val) {

  struct society_data *S = societyGet(name);
  if(S == NULL)
    return FALSE;

  if(val == FALSE)    S->open_leave = FALSE;
  else                S->open_leave = TRUE;

  societySave(S, societyFilename(S));
  return TRUE;
}


void listSocietyInfo(const char *name, struct char_data *ch) {

  struct society_data *S = societyGet(name);

  if(S == NULL) {
    send_to_char(ch, "%s is not a formal society.\r\n", name);
    return;
  }

  send_to_char(ch, "{c%s{n\n"
	           "%s\n"
 	           "%s\n"
	           "\n"
	           "%s\n",
	       S->name,
	       (S->open_join==FALSE)?"Players must be enrolled to join this society":
	                             "Players may join this society as they please",
	       (S->open_leave==FALSE)?"Players must be expelled to leave this society":
	                              "Players may leave this society as they please",
	       S->info);
}


void listSocietiesOfVictToChar(const struct char_data *vict, struct char_data *ch) {

  int i;
  bool society_found = FALSE;

  if(!IS_NPC(vict)) {
    for(i = 0; i < num_societies; i++) {

      if(societyMemberFromList(GET_IDNUM(vict), society[i]->moderators) ||
	 societyMemberFromList(GET_IDNUM(vict), society[i]->members)) {

	if(society_found == FALSE) {
	  society_found = TRUE;
	  send_to_char(ch, "Societies %s belongs to:\r\n", GET_NAME(vict));
	}
	send_to_char(ch, "  %s\r\n", society[i]->name);
      }
    }
  }

  if(!society_found)
    send_to_char(ch, "%s does not belong to any societies.\r\n", GET_NAME(vict));
}


void listSocieties(struct char_data *ch) {

  int i;

  if(num_societies < 1) {
    send_to_char(ch, "Presently, no societies have formally created.\r\n");
    return;
  }

  send_to_char(ch, "The list of societies on Aeonian Dreams:\r\n");
  for(i = 0; i < num_societies; i++)
    send_to_char(ch, "  %s\r\n", society[i]->name);

}
