//********************************************************************************
//
// This is the API for societies on Aeonian Dreams.
//
// Societies can be created in order for players to create their own rules to
// follow within some scope (i.e. the society). For instance, there could be a
// PKiller's society that allows all members to kill each other, or a Roleplayer's
// society where all members agree to roleplay whenever they are in the presence 
// of another member.
//
// Societies were implemented as a curiousity to see how players handle a world
// where they create and govern their own rules. Call me a sadist, but I'm 
// interested in what will happen...
//
// If this isn't your cup of tea, Societies can easily act as a fairly decent
// method for implementing clans. And, unlike most clan code out there, any given
// player can belong to any number of societies. If you decide to use our society
// code, please make note of it in your helpfile(s) relating to societies. Simply
// noting that this was orionally written by Alister of Aeonian Dreams would work
// splendid.
//
//  - Alister
//
//  Aeonian Dreams:
//    telnet://dreams.game-host.org:4000
//
//********************************************************************************

#define LVL_SOCIETY_MOD            108
#define DEFAULT_OPEN_JOIN          FALSE
#define DEFAULT_OPEN_LEAVE         FALSE
#define DEFAULT_SOCIETY_INFO       "little is known about this society, currently."
#define LIB_SOCIETY                LIB_ETC"societies/"
#define SOCIETY_LIST               LIB_SOCIETY"societies.soc"


//
// Boot up all the societies we have
//
void societyBoot();


//
// Reload a society from disk
// return true if soccessful. False otherwise
//
bool societyReload(const char *name);


//
// create a new society with the given name, and default info
// return false if a society already has that name
// returns true if the society is created
//
bool societyCreate(const char *name);


//
// Remove the society with the given name from the list
// of all societies
//
void societyDestroy(const char *name);


//
// Returns true if a society with the given name has formed
// returns false otherwise
//
bool isSociety(const char *name);


//
// Sets a character as applying to become a member of the
// given society.
//
// returns false if the character does not exist or is an NPC
// returns true if the member has already applied
// or if the application is successful
//
bool societyApply(const char *name, const struct char_data *ch);


//
// add a person to a society.
// returns false if the person is an NPC or does not exist
// returns true if the person was added or is already a member
//
bool societyInduct(const char *name, const struct char_data *ch);


//
// make a person a moderator of the society.
//
bool societyMakeModerator(const char *name, const struct char_data *ch);


//
// Expels a character from the society.
//
void societyExpel(const char *name, const struct char_data *ch);


//
// Decline a character's application to the society.
//
void societyDecline(const char *name, const struct char_data *ch);


//
// returns true if the character is a member of the society
// return false otherwise
//
bool societyIsMember(const char *name, const struct char_data *ch);


//
// returns true if the character is a moderator for the society
// (i.e. can induct and expell other members)
//
bool societyIsModerator(const char *name, const struct char_data *ch);


//
// returns true if the character is applying for the given
// society. returns false otherwise, or if the char is an NPC
//
bool societyIsApplicant(const char *name, const struct char_data *ch);


//
// returns true if anyone can join the society
// returns false if new members need to be inducted by a moderator
// 
bool societyOpenJoin(const char *name);


//
// return strue if anyone can feely leave the society
// return false if people need to be expelled by a moderator
//
bool societyOpenLeave(const char *name);


//
// Sets open_join to true or false
// returns true if successful and false otherwise
//
bool societySetOpenJoin(const char *name, const int val);


//
// Sets open_leave to true or false
// returns true if successful and false otherwise
//
bool societySetOpenLeave(const char *name, const int val);


//
// list all info for the given society to a character
//
void listSocietyInfo(const char *name, struct char_data *ch);


//
// send a list of all societies to the character
//
void listSocieties(struct char_data *ch);


//
// List all of the societies vict belongs to, to char
//
void listSocietiesOfVictToChar(const struct char_data *vict, struct char_data *ch);
